@@ -8,7 +8,11 @@ from metaflow.plugins.cards.card_modules.components import (
     Markdown,
     VegaChart,
     ProgressBar,
+    ValueBox,
     PythonCode,
+    EventsTimeline,
+    JSONViewer,
+    YAMLViewer,
 )
 from metaflow.plugins.cards.card_modules.basic import (
     DefaultCard,
