@@ -49,10 +49,6 @@ if TYPE_CHECKING:
 
 #     return tracefunc_closure
 
-CONFIG_FILE = os.path.join(
-    os.path.dirname(os.path.abspath(__file__)), "CONFIG_PARAMETERS"
-)
-
 ID_PATTERN = re.compile(r"^[a-zA-Z_][a-zA-Z0-9_]*$")
 
 UNPACK_KEY = "_unpacked_delayed_"
@@ -275,6 +271,9 @@ class ConfigValue(collections.abc.Mapping, dict):
             v = obj
         return v
 
+    def __reduce__(self):
+        return (self.__class__, (self.to_dict(),))
+
 
 class DelayEvaluator(collections.abc.Mapping):
     """
@@ -380,7 +379,7 @@ class DelayEvaluator(collections.abc.Mapping):
                 to_eval_expr,
                 self._globals or globals(),
                 {
-                    k: ConfigValue(v)
+                    k: ConfigValue(v) if v is not None else None
                     for k, v in flow_cls._flow_state.get(_FlowState.CONFIGS, {}).items()
                 },
             )
@@ -488,7 +487,6 @@ class Config(Parameter, collections.abc.Mapping):
         parser: Optional[Union[str, Callable[[str], Dict[Any, Any]]]] = None,
         **kwargs: Dict[str, str]
     ):
-
         if default is not None and default_value is not None:
             raise MetaflowException(
                 "For config '%s', you can only specify default or default_value, not both"
@@ -509,9 +507,7 @@ class Config(Parameter, collections.abc.Mapping):
         self._delayed_evaluator = None
 
     def load_parameter(self, v):
-        if v is None:
-            return None
-        return ConfigValue(v)
+        return ConfigValue(v) if v is not None else None
 
     def _store_value(self, v: Any) -> None:
         self._computed_value = v
