@@ -114,7 +114,7 @@ class MutableFlow:
         for name, value in self._flow_cls._flow_state.get(
             _FlowState.CONFIGS, {}
         ).items():
-            r = name, ConfigValue(value)
+            r = name, ConfigValue(value) if value is not None else None
             debug.userconf_exec("Mutable flow yielding config: %s" % str(r))
             yield r
 
@@ -347,8 +347,10 @@ class MutableFlow:
                     "Mutable flow adding flow decorator '%s'" % deco_type
                 )
 
+            # self._flow_cls._flow_decorators is a  dictionary of form :
+            # <deco_name> : [deco_instance, deco_instance, ...]
             existing_deco = [
-                d for d in self._flow_cls._flow_decorators if d.name == flow_deco.name
+                d for d in self._flow_cls._flow_decorators if d == flow_deco.name
             ]
 
             if flow_deco.allow_multiple or not existing_deco:
@@ -365,11 +367,11 @@ class MutableFlow:
                     "Mutable flow overriding flow decorator '%s' "
                     "(removing existing decorator and adding new one)" % flow_deco.name
                 )
-                self._flow_cls._flow_decorators = [
-                    d
+                self._flow_cls._flow_decorators = {
+                    d: self._flow_cls._flow_decorators[d]
                     for d in self._flow_cls._flow_decorators
-                    if d.name != flow_deco.name
-                ]
+                    if d != flow_deco.name
+                }
                 _do_add()
             elif duplicates == MutableFlow.ERROR:
                 # If we error, we raise an exception
@@ -470,7 +472,7 @@ class MutableFlow:
             % (len(old_deco_list) - len(new_deco_list))
         )
         if new_deco_list:
-            self._flow_cls._flow_decorators[deconame] = new_deco_list
+            self._flow_cls._flow_decorators[deco_name] = new_deco_list
         else:
             del self._flow_cls._flow_decorators[deco_name]
         return did_remove
